// Reset.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"


BOOL ResetUsbI2cIo(HANDLE * phDevInstance, char * sDevSerialId);

int _tmain(int argc, _TCHAR* argv[]) {

	BYTE byBoards;
	HANDLE hDevInstance = INVALID_HANDLE_VALUE;     // device instance handle
	char sDevSerialId[9] = "";

	// display the application version information
    system("cls");        // clear the console screen
	printf("\nDeVaSys USB-I2C/IO Reset Application 1.00\n");
	printf("Copyright DeVaSys 2009\n\n");

	if( !CheckDllAndDrvVersions(0x0304, 0x0300) ) {
		// the user's P.C. is not running appropriate versions of driver and dll
		printf("CRITICAL ERROR - System is not running a supported version of driver or dll\n");
		printf("No further operations are possible, exiting\n");
		printf("Press any key to continue\n");
		while (!kbhit());
		return FALSE;
	}

	byBoards = DAPI_GetDeviceCount( "UsbI2cIo" );

	if(byBoards == 0) {
		printf("CRITICAL ERROR - No USB-I2C/IO boards detected, exiting\n");
		printf("Press any key to continue\n");
		while (!kbhit());
		return 0;
	}
	else if(byBoards > 1) {
		printf("WARNING - Multiple USB-I2C/IO boards detected, using instance zero\n");
	}

	hDevInstance = DAPI_OpenDeviceInstance("UsbI2cIo", 0);
	if(hDevInstance == INVALID_HANDLE_VALUE) {
		// we were unable to open a handle to the USB-I2C/IO board
		printf("CRITICAL ERROR - Unable to open a handle to the USB-I2C/IO board\n");
		printf("Without a valid device handle, no further operations are possible, exiting\n");
		printf("Press any key to continue\n");
		while (!kbhit());
		return FALSE;
	}

	if( !CheckBoardFirmwareVersion(&hDevInstance, 0x0300, 0x0999) ) {
		printf("CRITICAL ERROR - Board is not running a supported version of firmware\n");
		printf("No further operations are possible, exiting\n");
		printf("Press any key to continue\n");
		while (!kbhit());
		return FALSE;
    }

	if( !DAPI_GetSerialId(hDevInstance, sDevSerialId) ) {
		// we were unable to fetch a serial number from the USB-I2C/IO board
		printf("CRITICAL ERROR - Unable to fetch a Serial ID from the USB-I2C/IO board\n");
		printf("Without a valid serial number, no further operations are possible, exiting\n");
		printf("Press any key to continue\n");
		while (!kbhit());
		return FALSE;
	}
	else {
		printf(" Board SerialId   = %s\n\n", sDevSerialId);
	}
	// if we make it to here, dll, driver, and firmware versions are all ok, and we have an open handle to a board.
  
	ResetUsbI2cIo(&hDevInstance, sDevSerialId);

	// close the handle
	DAPI_CloseDeviceInstance(hDevInstance);

	printf("\nPress any key to continue\n");
	while (!kbhit());
	return 1;
}

BOOL ResetUsbI2cIo(HANDLE * phDevInstance, char * sDevSerialId) {

	BYTE byRequest;
	WORD wValue;
	WORD wIndex;  // MSB(wSetupIndex) contains System Command Index
	WORD wLength;

	BOOL bRetVal;
	BYTE byGetData;

	byRequest = 0xE4;    // 0xE4 = VR_System
	wIndex = 0x0400;     // MSB = Reset, LSB = unused
	wValue = 0x0000;     // MSB = unused, LSB = unused
	wLength = 0;         // no data phase

	printf(" Resetting board\n");

	bRetVal = DAPI_SetVendorRequest(*phDevInstance, byRequest, wValue, wIndex, wLength, NULL);

	// strange case here... if it succeeds... it didn't work... it still may not have worked
	// this is because the board resets...  so it does not finish the transaction where the reset
	// was commanded.
	// also... the board will act as if was physically unplugged then replugged... which means
	// any existing handle to the board will be invalid (so we have to open a new one)

	// close the handle
	DAPI_CloseDeviceInstance(*phDevInstance);

	// need to wait for Windows to detect board removal, unload driver instance, detect new device, load new driver instance, etc.
	// (reset looks like unplug/replug)
	printf(" Pausing 7 seconds to allow for board reset\n");
	Sleep(7000);

	// open handle to same board using it's serial ID
	*phDevInstance = DAPI_OpenDeviceBySerialId("UsbI2cIo", sDevSerialId);
	if(*phDevInstance == INVALID_HANDLE_VALUE) {
		// we were unable to open a handle to the USB-I2C/IO board
		printf(" CRITICAL ERROR - Unable to open a handle to the USB-I2C/IO board\n");
		printf(" Without a valid device handle, no further operations are possible, exiting\n");
		//printf("Press any key to continue\n");
		//while (!kbhit());
		return FALSE;
	}

	// now we do a GetVendorRequest to get a byte value representing the source of the last reset (micro)
	wLength = 1;
	bRetVal = DAPI_GetVendorRequest(*phDevInstance, &byGetData, byRequest, wValue, wIndex, wLength);
	if(bRetVal) {
		// GetVendorRequest call succeeded, byGetData contains source of last reset
		// the value should be 0x10, indicating a software commanded reset
		if(byGetData == 0x10) {
			printf(" Board %s was successfully reset\n", sDevSerialId);
			printf(" Last reset source was: 0x%02X (software reset)\n", byGetData);
			return TRUE;
		}
		else {
			printf(" CRITICAL ERROR - Board %s was not successfully reset\n", sDevSerialId);
			printf("                  Last reset source was: 0x%02X (not a software reset)\n", byGetData);
			return FALSE;
		}
	}
	else {
		printf(" CRITICAL ERROR - GetVendorRequest() called returned FALSE, exiting\n");
		return FALSE;
	}
}




